/*
    OTP4U 0.8.1 - One Time Pad for you
    Copyright (C) 2003 Pierre Blanc
    Pierre Blanc: blanc_teutoburgo@yahoo.it
    http://teutoburgo.cjb.net
    http://it.geocities.com/teutoburgo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    or go to      http://www.gnu.org/copyleft/gpl.html
*/

package net.cjb.teutoburgo.otp4u.application;

import net.cjb.teutoburgo.otp4u.crypt.*;
import net.cjb.teutoburgo.otp4u.util.*;
import net.cjb.teutoburgo.otp4u.constants.*;
import net.cjb.teutoburgo.otp4u.io.*;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;

/**
 * <p>Title: OTP4U</p>
 * <p>Description: The main class for the command-line application
 * OTP4U (One Time Pad for you)</p>
 * <p>Copyright: Copyright (c) Pierre Blanc 2003</p>
 * <p>Company: Teutoburgo</p>
 * @author Pierre Blanc
 * revisions:
 *  05/29/03 - Karl Dinwiddie - some minor code clean-up so it can be  run
 *                              "out of the box" more easily.
 * @version 0.8.1
 */


public class OTP4U {
    public static String OTP4U_HOME = "";
    public static final String SEPARATOR=Constants.SEPARATOR,
	PUBLIC_KEY_FILE=Constants.PUBLIC_KEY_FILE, CFG_FILE=Constants.CFG_FILE,
	RANDOM_KEY_FILE=Constants.RANDOM_KEY_FILE, CIPHER=Constants.CIPHER,
	KEY_SMALL = Constants.KEY_SMALL , PLAIN=Constants.PLAIN,
	RANDOM_KEY=Constants.RANDOM_KEY , PUBLIC_KEY=Constants.PUBLIC_KEY,
	ENTROPY=Constants.ENTROPY , KEY4KEY=Constants.KEY4KEY,
	KEY4KEY_FILE=Constants.KEY4KEY_FILE,
	PSEUDO_RANDOM=Constants.PSEUDO_RANDOM,
	badParameter=Constants.BAD_PARAMETER, CONFIG = Constants.CONFIG;
    private int defaultLength=3000;
    KeyManager keyManager;
    Xor xor;
    Io io;
    Cipher cipher;
	static MessagesPrinter mp;
//    MessagesPrinter mp;
//    Random rnd = new Random();
    RandomGenerator rg;
    private long DEFAULT_DEVIATION=100;
    private String defaultFile="file",	stringLog="";
    private boolean isGUI=false;
    /**
     * OTP4U constructor comment.
     */
    public OTP4U() {
    }

    public OTP4U(boolean isGUI) {
	super();
	OTP4U_HOME=new File((new File("")).getAbsolutePath()).getParent()+
	    SEPARATOR;
	String temp = "";
	Properties prop = new Properties();
	FileInputStream fis=null, fis2=null;
	try {
	    fis = new FileInputStream(new File(OTP4U_HOME+CONFIG+CFG_FILE));
	    otp4uInit(CFG_FILE);
	} catch (FileNotFoundException fnfe){

				File file = new File("c:/otp4u_home.cfg");
			try {
				fis2 = new FileInputStream(file);
/*
	    try{
		fis2 = new FileInputStream(new File("c:/otp4u_home.cfg"));
*/	    } catch (Exception e){
		mp.sPrintLog("Error! Can't find c:/otp4u_home.cfg");
		mp.sPrintLog("Under Windows copy it from patch4win/otp4u_home.cfg");
		System.exit(-1);
	    }
	    try{
		prop.load(fis2);
		OTP4U_HOME=prop.getProperty("OTP4U_HOME");
		otp4uInit(CFG_FILE);
	    }catch(IllegalArgumentException ie){
		ie.printStackTrace();
	    }catch(ClassNotFoundException cnfe){
		cnfe.printStackTrace();

	    } catch (FileNotFoundException e){
		e.printStackTrace();
		mp.sPrintLog("Using OTP4U_HOME="+OTP4U_HOME);
		mp.sPrintLog("Set the OTP4U_HOME");
		System.exit(-1);
	    } catch(IOException ioe){
		ioe.printStackTrace();
		System.exit(-1);
	    } catch (Exception e){
		e.printStackTrace();
	    }
	} catch (IOException ioe) {
	    ioe.printStackTrace();
	    System.exit(-1);
	    //	} catch (NumberFormatException e){

	} catch (Exception e){
	    e.printStackTrace();
	}
	this.isGUI=isGUI;
    }
    /**
     *	Used by the contructor and by a GUI method to load the properties
     *	in the configuration file
     *	@param fileName the name of the configuration file
     */
    public void otp4uInit(String fileName) throws FileNotFoundException,
	IllegalArgumentException, IOException, NumberFormatException,
	ClassNotFoundException, NoSuchMethodException, Exception {

	String temp = "";
	Properties prop = new Properties();
	FileInputStream fis=null, fis2=null;
	xor= new Xor();
	// System.out.println("\n---- honme "+OTP4U_HOME);
	io = new Io(OTP4U_HOME, isGUI);
	cipher = new Cipher(OTP4U_HOME, isGUI);
        rg = new RandomGenerator(OTP4U_HOME, isGUI);
	mp = new MessagesPrinter(isGUI);
	fis = new FileInputStream(new File(OTP4U_HOME+CONFIG+fileName));
	prop.load(fis);
	temp = prop.getProperty("keymanager");
	Boolean b=new Boolean(isGUI);
	Class[] parameters = {OTP4U_HOME.getClass(), b.getClass()};
	Constructor constr=Class.forName(
          temp).getConstructor(parameters);
	Object[] initArgs={OTP4U_HOME, b};
	keyManager = (KeyManager)constr.newInstance(initArgs);
	temp = prop.getProperty("defaultfile","file");
	defaultFile=temp;
	temp = prop.getProperty("randomFileDefaultLength","3");
        defaultLength=Integer.parseInt(temp)*1024;
        mp.sPrintLog("Read configuration from file "+
		     OTP4U_HOME+CONFIG+fileName);
    }

    /**
     * Used to generate a PSEUDO-random file in the entropySource directory
     * or in the key4key directory.
     * Use it to test the functionalities of OTP4U.
     * @param pathName the path name of the pseudo-random file
     * @param fileName the file name
     * @param length the file length
     */
    public void generateRandomFile(String pathName, String fileName,
				   int length){

    rg.generateRandomFile(pathName, fileName, length);
                                   /*	String generated="";
	int fLength=fileName.length();
	int max=0;
	byte[] randomBytes = new byte[length];
	rnd.nextBytes(randomBytes);
	if(fileName.equals(PSEUDO_RANDOM)){
	    File f = new File(pathName);
	    File[] list = f.listFiles();
	    try{
		for (int i=0; i<list.length; i++){
		    if (!(fLength>list[i].getName().toString().length())){
			if(list[i].getName().toString().substring(
			  0,fLength).equals(fileName))
			    max++;
		    }
		}
	    }catch (Exception e){
		e.printStackTrace();
	    }
	    String progressive=Integer.toString(max+1);
	    generated=pathName+fileName+progressive+".rnd";
	} else
	    generated=pathName+fileName;
	try{
	    io.writeFile(generated,randomBytes);
	}catch (Exception e){
	    e.printStackTrace();
	}
	mp.sPrintLog("Generated "+length+" random bytes in file "+generated);*/
    }

    /**
     * Performs decryption if the files of ciphertext and randomKey are found <br>
     * Writes the plaintext array in its directory
     * @param pathName The pathName of the ciphertext
     */
    public void doDecryption(String pathName) {
	cipher.decipher(pathName);
    }

    /**
     * Performs encryption if the files of plaintext and randomKey are found <br>
     * Writes the ciphertext array ciphertext in its directory
     * @param pathName The pathname of the plaintext
     */
    public void doEncryption(String pathName) {
	cipher.encipher(pathName);
    }

    public String getDefaultFile(){
	return defaultFile;
    }

    public int getDefaultLength(){
	return defaultLength;
    }

    public int getRandomKeySize(){
	try{
	    byte[] randomKey = io.readFile(OTP4U_HOME +RANDOM_KEY +
					   RANDOM_KEY_FILE);
	    return randomKey.length;
	}catch (Exception e){
	    e.printStackTrace();
	    return 0;
	}
    }

    public byte[][] initArrays(String pathName) throws IOException {
	byte[][] arrays=new byte[2][];
	byte[] fileToXOR=null;

	fileToXOR = io.readFile(pathName);
	byte[] randomKeyBytes = io.readFile(OTP4U_HOME + RANDOM_KEY +
					    RANDOM_KEY_FILE);
	arrays[0]=fileToXOR;
	arrays[1]=randomKeyBytes;

	return arrays;
    }

    public static void main(String args[]) {
	OTP4U otp4u = new OTP4U(false);
	mp = new MessagesPrinter();
//	MessagesPrinter mp = new MessagesPrinter();
        mp.printCopyright();
	int argsLen = args.length;
	String fileName = otp4u.getDefaultFile();
        int length=otp4u.getDefaultLength();

	if (argsLen == 0) {
	    mp.printHelp();
	}
	else if (argsLen == 1) {
	    if (args[0].equalsIgnoreCase("e"))
		otp4u.doEncryption(fileName);
	    else if (args[0].equalsIgnoreCase("d"))
		otp4u.doDecryption(fileName);
	    else if (args[0].equalsIgnoreCase("h")
		     || args[0].equalsIgnoreCase("help")
		     || args[0].equalsIgnoreCase("-h")
		     || args[0].equalsIgnoreCase("--help"))
		mp.printHelp();
            else if (args[0].equalsIgnoreCase("g"))
                otp4u.generateRandomFile(OTP4U_HOME+ENTROPY, PSEUDO_RANDOM,
					 length);
	    else if (args[0].equalsIgnoreCase("w")){
                mp.printNoWarranty();
            } else if (args[0].equalsIgnoreCase("p")){
                otp4u.writePublicKey();
            } else if (args[0].equalsIgnoreCase("r")){
                otp4u.writeRandomKey();
            } else if (args[0].equalsIgnoreCase("test")){
		try{
		    /*   Io io = new Io();
			 byte[] x = io.readFile(OTP4U_HOME + RANDOM_KEY + KEY4KEY +
			 KEY4KEY_FILE);
			 byte[] y = io.readFile(OTP4U_HOME + RANDOM_KEY + KEY4KEY +
			 "test.rnd");
			 io.writeFile(OTP4U_HOME + RANDOM_KEY + KEY4KEY+"autoKey.rnd",
			 otp4u.autoKey(x, y));*/
		} catch(Exception e){
		    e.printStackTrace();
		}
		/*            } else if (args[0].equalsIgnoreCase("t")){//testPermutation
			      byte[] pr={1,2,3,4,5,6,7,8,9,10,11};
			      otp4u.getKey4Message(pr,pr);*/
            } else
		System.out.println(badParameter);
	} else if (argsLen == 2)
	    if (args[0].equalsIgnoreCase("e"))
		otp4u.doEncryption(args[1]);
	    else if (args[0].equalsIgnoreCase("d"))
		otp4u.doDecryption(args[1]);
            else if (args[0].equalsIgnoreCase("g")){
		try{
		    length=Integer.parseInt(args[1])*1024;
		} catch (NumberFormatException nfe){
		    nfe.printStackTrace();
		    System.exit(-1);
		}
		otp4u.generateRandomFile(OTP4U_HOME+ENTROPY, PSEUDO_RANDOM, length);
            }
            else if (args[0].equalsIgnoreCase("gk4k0")){
		try{
		    length=Integer.parseInt(args[1])*1024;
		} catch (NumberFormatException nfe){
		    nfe.printStackTrace();
		    System.exit(-1);
		}
		otp4u.generateRandomFile(OTP4U_HOME + RANDOM_KEY + KEY4KEY,
					 KEY4KEY_FILE, length);
            }
	    else
		System.out.println(badParameter);
	else
	    System.out.println("Wrong number of parameters "+argsLen);
    }

    /**
     *	Used to write the "public" key
     */
    public void writePublicKey(){
	try{
	    keyManager.writePublicKey();

	}catch(Exception e){
	    e.printStackTrace();
	}
    }


    /**
     *	Used to write the random key (composed of K4Kn and K4Mn)
     */
    public void writeRandomKey(){
	try{
	    keyManager.writeRandomKey();

	} catch (IOException ioe){
	    ioe.printStackTrace();
	    mp.sCatchException();
	}
    }


}
